// CLI__Demo3.cpp: Hauptprojektdatei.

#include "stdafx.h"
#include <iostream>

using namespace std;
using namespace System;

using namespace System::Collections::Generic;
using namespace System::ComponentModel;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Text;
using namespace System::Windows::Forms;
using namespace PCBI::Automation;
using namespace System::Drawing::Drawing2D; 
static void CreateDemoCMPLayer(IPCBIWindow^ window, PCBI::Automation::IFilter^ filter);
void CreateDemoODBLayer(PCBI::Automation::IFilter^ filter);

int main()
{
	IAutomation::IAutomationInit();
			
			
				IPCBIWindow^ window;
				
					//create a new instance of PCB-Investigator
				
					window = PCBI::Automation::IAutomation::CreateNewPCBIWindow(true,false);
					if (window == nullptr)
					{
						MessageBox::Show("Error:\n" + PCBI::Automation::IAutomation::GetErrorLog(), "Attention", MessageBoxButtons::OK, MessageBoxIcon::Error);
						return -1;
					}
					
				
				PCBI::Automation::IFilter^ filter = gcnew PCBI::Automation::IFilter(window);

				// create new job
				
				
				String^ JobFullPath = filter->CreateAndLoadEmptyJob("D:\\tmp\\ThirdTestJob");
				

				CreateDemoODBLayer(filter);



				CreateDemoCMPLayer(window, filter);


				//update matrix
				IMatrix^ matrix = window->GetMatrix();
				if (matrix != nullptr)
					matrix->UpdateDataAndList();

				window->SaveJob();
			}

			 static void CreateDemoCMPLayer(IPCBIWindow^ window, PCBI::Automation::IFilter^ filter)
			{
				ICMPLayer^ parentCMP = window->GetCurrentStep()->GetCMPLayer(true);
				if (parentCMP == nullptr)
					parentCMP = filter->CreateEmptyCMPLayer("cmp_top", "gerber"); //in this example the stepname is "gerber"
				ICMPObject^ component = filter->CreateComponent(parentCMP);

				IPackageSpecifics^ pack = filter->CreatePackage("testPack"); //one package for the demo-cmp

				GraphicsPath^ gp = gcnew GraphicsPath();
				
				gp->AddLine( Point(100, 100),  Point(100, 300));
				gp->AddLine(Point(100, 300),  Point(300, 300));
				gp->AddLine(Point(300, 300), Point(300, 100));
				gp->AddLine( Point(300, 100), Point(100, 100));

				pack->SetGraphicsPath(gp);


				pack->AddEllipsePin(PointF(101, 101), 35, "PinEl1");
				pack->AddRectanglePin( RectangleF(279, 269, 40, 50), "PinRe1");

				for (int i = 0; i < 5; i++)
				{
					
					ISurfaceSpecifics^ outline = pack->CreatePolygonPinSurface();
					outline->StartPolygon(false, PointF(20, 50));
					outline->AddLine( PointF(20, 50),  PointF(20, 70));
					outline->AddLine(PointF(20, 70),  PointF(35, 70));

					outline->AddArc( PointF(35, 70),  PointF(40, 65),  PointF(35, 65), true);

					outline->AddLine( PointF(40, 65),  PointF(40, 50));
					outline->AddLine(PointF(40, 50), PointF(20, 50));
					outline->EndPolygon(false);
					pack->AddPolygonPin( PointF(100 + i * 20, 150 + 1 * 5), outline, "PinPo" + i); //add the polygon pin with nr
				}


				IObjectSpecifics^ specifics = component->GetSpecifics();
				IComponentSpecifics^ spec = (IComponentSpecifics^)specifics;
				spec->Height = 7;
				spec->Location = PointF(1000, 500);
				spec->PartName = "partName2";
				spec->Value = "value2";
				spec->Attributes = "attributes2";
				spec->Reference = "Ref2";
				spec->PlacedTop = true;

				component->SetSpecifics(spec, pack);
			}

			/// <summary>
			/// An example to create an new layer and add some objects to show how to handle simple properties.
			/// </summary>
			/// <param name="filter">The IFilter object to create the elements.</param>
			 void CreateDemoODBLayer(PCBI::Automation::IFilter^ filter)
			 {
				 IODBLayer^ layer = filter->CreateEmptyODBLayer((String^)"layer1", (String^)"gerber",false);

				 //tooldefinitions
				 int index1 = IFilter::AddToolDefinitionRound(layer, 10, 0,false);
				 int index2 = IFilter::AddToolDefinitionRect(layer, 20, 1, 20, 50, true);
				 int index3 = IFilter::AddToolDefinitionDonut(layer, 50, 25, 2);
				 int index4 = IFilter::AddToolDefinitionOct(layer, 40, 40, 40, 5, 3);

				 //create an arc object
				 IODBObject^ arc = filter->CreateArc(layer);
				 IArcSpecifics^ specificsArc = gcnew IArcSpecifics();
				 specificsArc->Start =  PointF(100, 100);
				 specificsArc->End =  PointF(100, 300);
				 specificsArc->Center = PointF(100, 200);
				 specificsArc->ClockWise = true;
				 specificsArc->Positive = true;
				 specificsArc->Diameter = 10;
				 arc->SetSpecifics(specificsArc, 0);


				 //create three pads
				 IODBObject^ pad = filter->CreatePad(layer);
				 IPadSpecifics^ padInfos = gcnew IPadSpecifics();
				 padInfos->Location =  PointF(400, 400);
				 padInfos->Positive = (true);
				 pad->SetSpecifics(padInfos, index1); //its possible to use the same index more than one time

				 IODBObject^ pad2 = filter->CreatePad(layer);
				 padInfos = gcnew IPadSpecifics();
				 padInfos->Location =  PointF(300, 400);
				 padInfos->Positive = (true);
				 pad2->SetSpecifics(padInfos, index2);

				 IODBObject^ pad3 = filter->CreatePad(layer);
				 padInfos = gcnew IPadSpecifics();
				 padInfos->Location =  PointF(200, 400);
				 padInfos->Positive = (true);
				 pad3->SetSpecifics(padInfos, index3);

			 } 
